/******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation
 * All rights reserved.
 * This program and the accompanying materials
 * are made available under the terms of the BSD License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Contributors:
 *     IBM Corporation - initial implementation
 *****************************************************************************/


#ifndef _FCNTL_H
#define _FCNTL_H

#define O_RDONLY    00
#define O_WRONLY    01
#define O_RDRW      02

#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

#endif /* fcntl.h */
