/******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation
 * All rights reserved.
 * This program and the accompanying materials
 * are made available under the terms of the BSD License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Contributors:
 *     IBM Corporation - initial implementation
 *****************************************************************************/

#ifndef __STAGE2_H
#define __STAGE2_H

#ifndef __ASSEMBLER__

#include <stddef.h>

void u4memInit(void);

#endif		/* __ASSEMBLER__ */
#endif		/* __STAGE2_H */
