\ *****************************************************************************
\ * Copyright (c) 2004, 2011 IBM Corporation
\ * All rights reserved.
\ * This program and the accompanying materials
\ * are made available under the terms of the BSD License
\ * which accompanies this distribution, and is available at
\ * http://www.opensource.org/licenses/bsd-license.php
\ *
\ * Contributors:
\ *     IBM Corporation - initial implementation
\ ****************************************************************************/

cr .(   Copyright (c) char ) emit .(  2004, 2011 IBM Corporation All rights reserved.)
cr .(   This program and the accompanying materials are made available)
cr .(   under the terms of the BSD License available at)
cr .(   http://www.opensource.org/licenses/bsd-license.php)
